﻿using System;
using Newtonsoft.Json;

namespace Geekbot.Core.Logger
{
    public class GeekbotLogger : IGeekbotLogger
    {
        private readonly bool _logAsJson;
        private readonly Serilog.ILogger _logger;
        private readonly JsonSerializerSettings _serializerSettings;

        public GeekbotLogger(RunParameters runParameters)
        {
            _logAsJson = !string.IsNullOrEmpty(runParameters.DatadogApiKey) || runParameters.LogJson;
            _logger = LoggerFactory.CreateLogger(runParameters);
            _serializerSettings = new JsonSerializerSettings
            {
                ReferenceLoopHandling = ReferenceLoopHandling.Serialize,
                NullValueHandling = NullValueHandling.Include
            };
            Information(LogSource.Geekbot, "Using GeekbotLogger");
        }

        public void Trace(LogSource source, string message, object extra = null)
            => _logger.Verbose(CreateLogString("Trace", source, message, null, extra));

        public void Debug(LogSource source, string message, object extra = null)
            => _logger.Debug(CreateLogString("Debug", source, message, null, extra));

        public void Information(LogSource source, string message, object extra = null)
            => _logger.Information(CreateLogString("Information", source, message, null, extra));

        public void Warning(LogSource source, string message, Exception stackTrace = null, object extra = null)
            => _logger.Warning(CreateLogString("Warning", source, message, stackTrace, extra));

        public void Error(LogSource source, string message, Exception stackTrace, object extra = null)
            => _logger.Error(stackTrace, CreateLogString("Error", source, message, stackTrace, extra));

        public Serilog.ILogger GetLogger() => _logger;

        public bool LogAsJson() => _logAsJson;

        private string CreateLogString(string type, LogSource source, string message, Exception stackTrace = null, object extra = null)
        {
            if (_logAsJson)
            {
                var logObject = new GeekbotLoggerObject
                {
                    Timestamp = DateTime.Now,
                    Type = type,
                    Source = source,
                    Message = message,
                    StackTrace = stackTrace,
                    Extra = extra
                };
                return JsonConvert.SerializeObject(logObject, Formatting.None, _serializerSettings);
            }

            if (source != LogSource.Message) return $"[{source}] - {message}";

            var m = (MessageDto) extra;
            return $"[{source}] - [{m?.Guild.Name} - {m?.Channel.Name}] {m?.User.Name}: {m?.Message.Content}";
        }
    }
}