﻿using System;
using Serilog;
using Serilog.Events;
using Serilog.Sinks.Datadog.Logs;

namespace Geekbot.Core.Logger
{
    public class LoggerFactory
    {
        public static ILogger CreateLogger(RunParameters runParameters)
        {
            var logger = new LoggerConfiguration();
            var minLevel = runParameters.Verbose ? LogEventLevel.Verbose : LogEventLevel.Information;
            if (!string.IsNullOrEmpty(runParameters.DatadogApiKey))
            {
                // 2nd error logger
                var errorLogger = new LoggerConfiguration().WriteTo.Console().CreateLogger();
                errorLogger.Information("Enabling Datadog Logger");
                
                logger.WriteTo.DatadogLogs(
                    apiKey: runParameters.DatadogApiKey,
                    source: "GeekbotLogger",
                    service: "Geekbot",
                    host: Environment.MachineName,
                    configuration: new DatadogConfiguration()
                    {
                        Url = "https://http-intake.logs.datadoghq.eu",
                        Port = 443,
                        UseSSL = true,
                        UseTCP = false
                    },
                    logLevel: minLevel,
                    exceptionHandler: exception =>
                    {
                        var cannotSendLogEventException = exception as CannotSendLogEventException;
                        errorLogger.Error(cannotSendLogEventException, "Datadog Error");
                    }
                );
            }
            else if (runParameters.LogJson)
            {
                logger.WriteTo.Console(restrictedToMinimumLevel: minLevel, outputTemplate: "{Message:lj}{NewLine}");
            }
            else
            {
                logger.WriteTo.Console(restrictedToMinimumLevel: minLevel);
            }

            return logger.CreateLogger();
        }

        // public static NLog.Logger CreateNLog(RunParameters runParameters)
        // {
        //     var config = new LoggingConfiguration();
        //     var minLevel = runParameters.Verbose ? LogLevel.Trace : LogLevel.Info;
        //
        //     if (!string.IsNullOrEmpty(runParameters.SumologicEndpoint))
        //     {
        //         Console.WriteLine("Logging Geekbot Logs to Sumologic");
        //         config.LoggingRules.Add(
        //             new LoggingRule("*", minLevel, LogLevel.Fatal,
        //                 new SumoLogicTarget()
        //                 {
        //                     Url = runParameters.SumologicEndpoint,
        //                     SourceName = "GeekbotLogger",
        //                     Layout = "${message}",
        //                     UseConsoleLog = false,
        //                     OptimizeBufferReuse = true,
        //                     Name = "Geekbot"
        //                 })
        //         );
        //     }
        //     else if (runParameters.LogJson)
        //     {
        //         config.LoggingRules.Add(
        //             new LoggingRule("*", minLevel, LogLevel.Fatal,
        //                 new ConsoleTarget
        //                 {
        //                     Name = "Console",
        //                     Encoding = Encoding.UTF8,
        //                     Layout = "${message}"
        //                 }
        //             )
        //         );
        //     }
        //     else
        //     {
        //         config.LoggingRules.Add(
        //             new LoggingRule("*", minLevel, LogLevel.Fatal,
        //                 new ColoredConsoleTarget
        //                 {
        //                     Name = "Console",
        //                     Encoding = Encoding.UTF8,
        //                     Layout = "[${longdate} ${level:format=FirstCharacter}] ${message} ${exception:format=toString}"
        //                 }
        //             )
        //         );
        //     }
        //
        //     var loggerConfig = new LogFactory {Configuration = config};
        //     return loggerConfig.GetCurrentClassLogger();
        // }
    }
}