﻿using System;
using Geekbot.Core.Logger;
using Microsoft.Extensions.Logging;
using Serilog.Events;

namespace Geekbot.Web.Logging
{
    public class AspLogger : ILogger
    {
        private readonly string _categoryName;
        private readonly IGeekbotLogger _geekbotLogger;
        
        public AspLogger(string categoryName, IGeekbotLogger geekbotLogger)
        {
            geekbotLogger.Trace(LogSource.Api, $"Adding {categoryName}");
            _categoryName = categoryName;
            _geekbotLogger = geekbotLogger;
        }
        
        public void Log<TState>(LogLevel logLevel, EventId eventId, TState state, Exception exception, Func<TState, Exception, string> formatter)
        {
            switch (logLevel)
            {
                case LogLevel.Trace:
                    _geekbotLogger.Trace(LogSource.Api, $"{eventId.Id} - {_categoryName} - {state}");
                    break;
                case LogLevel.Debug:
                    _geekbotLogger.Debug(LogSource.Api, $"{eventId.Id} - {_categoryName} - {state}");
                    break;
                case LogLevel.Information:
                    _geekbotLogger.Information(LogSource.Api, $"{eventId.Id} - {_categoryName} - {state}");
                    break;
                case LogLevel.Warning:
                    _geekbotLogger.Warning(LogSource.Api, $"{eventId.Id} - {_categoryName} - {state}", exception);
                    break;
                case LogLevel.Error:
                case LogLevel.Critical:
                    _geekbotLogger.Error(LogSource.Api, $"{eventId.Id} - {_categoryName} - {state}", exception);
                    break;
                case LogLevel.None:
                    break;
                default:
                    throw new ArgumentOutOfRangeException(nameof(logLevel));
            }
        }

        public bool IsEnabled(LogLevel logLevel)
        {
            return !_geekbotLogger.LogAsJson() && _geekbotLogger.GetLogger().IsEnabled(ToGeekbotLogLevel(logLevel));
        }

        public IDisposable BeginScope<TState>(TState state)
        {
            return null;
        }
        
        private static LogEventLevel ToGeekbotLogLevel(LogLevel level)
        {
            return level switch
            {
                LogLevel.Trace => LogEventLevel.Verbose,
                LogLevel.Debug => LogEventLevel.Debug,
                LogLevel.Information => LogEventLevel.Information,
                LogLevel.Warning => LogEventLevel.Warning,
                LogLevel.Error => LogEventLevel.Error,
                LogLevel.Critical => LogEventLevel.Fatal,
                _ => throw new ArgumentOutOfRangeException(nameof(level))
            };
        }
    }
}